
/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : Dec. 2006

 File          : LIN_Test_HVintFIQ.c

 Hardware      : Applicable to ADuC703x

 Description   : Wake-up Timer causes part to power-up every 8s.
 				 the timer is configured in Hr:Min:Sec:Hundredths format
				
				Additional (Gilbert): Can be woke up by LIN during power down mode.

				 T2LD:
				 bits 31-24 represent the hours
				 bits 21-16 represent the minutes
				 bits 13-8 represent the secondes
				 bits 6-0 represent the hundredths

				 The debugger cannot be used when the part is placed in power down mode.
		
*********************************************************************/

#include <ADuC7038.h>

void LIN_INIT(void);
void My_FIQ_Function(void);
void My_IRQ_Function(void);

void delay(int);
char HVstatus;
int iFIQ_Source,itemp1,itemp2,itemp3, iIRQSIG;

int main()
{
	LIN_INIT();
 	GPDAT = 0x0f000000;		// Set all bits to outputs

	FIQ = My_FIQ_Function; 	// Initialise Interrupt Routine	 for LIN
	FIQEN = 0x400;			// Enable HV interrupt
//	delay(200);
//	iIRQSIG = FIQSIG;	// Save a temporary copy of IRQSTA	
						// Count down, Enable Timer 2, hours mode
	GPCLR = 0x000F0000;		// Set all bits to Zero

	while(1)
											
	{

	}				// Infinite Loop
}


void My_FIQ_Function() 
{

//	iIRQSIG = FIQSIG;
	iFIQ_Source = FIQSTA;			// Save a temporary copy of IRQSTA	

		do
		{
			HVstatus = HVCON;
		}
		while(HVstatus & 0x1); 		// Wait until busy ia deaSSERTED

	iIRQSIG = HVDAT;
	LINCON = 0x00;
	do
	{
	 HVstatus = HVCON;
  	}
	while (!(HVstatus & 0x2));		// transmit command  GOOD
	do										
	{
		HVDAT = 0x00; 						// Enable Normal LIN TX mode / reenable short circuit
		HVCON = 0x08; 						// Write to Config0
		do
		{
			HVstatus = HVCON;
		}
		while(HVstatus & 0x1); 				// Wait until command is finished
	}
	while (!(HVstatus & 0x4));				// transmit command is correct

	LIN_INIT();
	GPCLR = 0x003F0000;		// Set all bits to Zero
		
	 return;
}


void delay (int length)
{
	while (length >=0)
    	length--;
}

void LIN_INIT(void )
{

	GPCON = 0x100000;						// enable LIN Rx and LIN Tx on GPIO
	LINCON = 0x1800;							// Enable LIN

	do										
	{
		HVDAT = 0x07; 						// Enable Normal LIN TX mode / reenable short circuit
		HVCON = 0x08; 						// Write to Config0
		do
		{
			HVstatus = HVCON;
		}
		while(HVstatus & 0x1); 				// Wait until command is finished
	}
	while (!(HVstatus & 0x4));				// transmit command is correct


// LINWU = 0x0010;
// LINBK = 0x030;							// Set number of 131kHz periods to generate a Break Interrupt
											// 0x3F / 131kHz ~ 480us which is just over 9.5 TBits.
											// According to LIN V2.0, a slave with a clock accuracy of <2% may
											// use a break detection of 9.5 bit, otherwise a break detection of
											// 11 bits is required
											// This is for 19200bps
}

